#define MBTK_NO_INCLUDE_WIDGETS
#include <libmbtk/widget.h>

static sdl2_win_t *win2sw(mbtk_window_t *win)
{
	while(win->in_widget != NULL)
		win = win->in_widget->win;
	return (sdl2_win_t *)win;
}

#define MAYBE_SKIP_RENDER(skip_stmt) \
do { \
	sdl2_win_t *ssw = (sdl2_win_t *)win; \
	if (ssw->skip_render) { skip_stmt; } \
} while(0)

static int mbtk_sdl2_draw_color(mbtk_window_t *win, int r, int g, int b, int a)
{
	sdl2_win_t *sw = win2sw(win);
	MAYBE_SKIP_RENDER(return 0);

	SDL_SetRenderDrawColor(sw->renderer, (r & 0xff), (g & 0xff), (b & 0xff), (a & 0xff));
	sw->clr.r = r;
	sw->clr.g = g;
	sw->clr.b = b;
	sw->clr.a = a;
	return 0;
}

static void mbtk_sdl2_draw_fillrect(mbtk_window_t *win, int x1, int y1, int x2, int y2)
{
	sdl2_win_t *sw = win2sw(win);
	SDL_Rect r;
	MAYBE_SKIP_RENDER(return);

	r.x = x1;
	r.y = y1;
	r.w = x2 - x1 + 1;
	r.h = y2 - y1 + 1;

	SDL_RenderFillRect(sw->renderer, &r);
}

static void mbtk_sdl2_draw_line(mbtk_window_t *win, int x1, int y1, int x2, int y2)
{
	sdl2_win_t *sw = win2sw(win);
	MAYBE_SKIP_RENDER(return);

	SDL_RenderDrawLine(sw->renderer, x1, y1, x2, y2);
}

static void mbtk_sdl2_draw_hline(mbtk_window_t *win, int x1, int y1, int x2)
{
	sdl2_win_t *sw = win2sw(win);
	MAYBE_SKIP_RENDER(return);

	SDL_RenderDrawLine(sw->renderer, x1, y1, x2, y1);
}

static void mbtk_sdl2_draw_vline(mbtk_window_t *win, int x1, int y1, int y2)
{
	sdl2_win_t *sw = win2sw(win);
	MAYBE_SKIP_RENDER(return);

	SDL_RenderDrawLine(sw->renderer, x1, y1, x1, y2);
}

static void mbtk_sdl2_set_win_viewport(sdl2_win_t *sw, mbtk_window_t *win)
{
	if (win->in_widget) {
		SDL_Rect r;
		r.x = win->in_widget->hvbox.x; r.y = win->in_widget->hvbox.y;
		r.w = win->in_widget->hvbox.w; r.h = win->in_widget->hvbox.h;
		SDL_RenderSetViewport(sw->renderer, &r);
	}
	else
		SDL_RenderSetViewport(sw->renderer, NULL); /* full window */
}

static void mbtk_sdl2_draw_flush(mbtk_window_t *win)
{
	sdl2_win_t *sw = win2sw(win);
	if (win->in_widget != NULL) {
		if (win->native_draw_post != NULL)
			win->native_draw_post(win, sw->renderer);

		/* restore parent viewport, run render present only when we are back at
		   window scope */
		mbtk_sdl2_set_win_viewport(sw, win->in_widget->win);
		return;
	}
	SDL_RenderPresent(sw->renderer);
}


static void mbtk_sdl2_draw_start(mbtk_window_t *win)
{
	sdl2_win_t *sw = win2sw(win);
	if (win->in_widget != NULL) {
		sdl2_win_t *ssw = (sdl2_win_t *)win;
		ssw->skip_render = 0;
		mbtk_sdl2_set_win_viewport(sw, win);
		if (win->native_draw_pre != NULL)
			ssw->skip_render = win->native_draw_pre(win, sw->renderer);
		return;
	}
	SDL_RenderClear(sw->renderer);
}



static int mbtk_sdl2_draw_set_font(mbtk_window_t *win, mbtk_font_id_t fontid)
{
	sdl2_win_t *sw = win2sw(win);
	if ((fontid.l == -1) || (fontid.p == NULL))
		return -1;

	MAYBE_SKIP_RENDER(return 0);

	sw->font = fontid.p;
	return 0;
}

static void mbtk_sdl2_draw_query_text(mbtk_window_t *win, int *w, int *ascent, int *descent, char *s, int slen)
{
	sdl2_win_t *sw = win2sw(win);
	int h;

	*ascent = TTF_FontAscent(sw->font->ttf);
	*descent = -TTF_FontDescent(sw->font->ttf);
	TTF_SizeText(sw->font->ttf, s, w, &h);
}

static void mbtk_sdl2_draw_text(mbtk_window_t *win, int x, int y, char *s, int slen, void **cache)
{
	int asc;
	sdl2_win_t *sw = win2sw(win);
	mbtk_sdl2_text_t *txt = *cache;
	MAYBE_SKIP_RENDER(return);

	if (txt == NULL) {
		txt = mbtk_sdl2_text_new(sw->fonts, sw->font, s, sw->clr);
		if (cache != NULL)
			*cache = txt;
	}

	asc = TTF_FontAscent(sw->font->ttf);
/*	dsc = -TTF_FontDescent(sw->font->ttf);*/

	mbtk_sdl2_text_render(sw->fonts, txt, x, y-asc);
}


static void mbtk_sdl2_draw_free_text(mbtk_window_t *win, void **cache)
{
	sdl2_win_t *sw = win2sw(win);

	if (*cache == NULL)
		return;

	mbtk_sdl2_text_destroy(sw->fonts, *cache);
	*cache = NULL;
}


static int mbtk_sdl2_draw_import_pixmap(mbtk_display_t *display, mbtk_pixmap_t *dst, unsigned char *pixels, int w, int h, int has_alpha)
{
	int x, y, dx = has_alpha ? 4 : 3;
	unsigned char *s, *d, *tmp = malloc(w*h*4);

	d = tmp;
	s = pixels;
	for(y = 0; y < h; y++) {
		for(x = 0; x < w; x++,s+=dx,d+=4) {
			unsigned char r = s[0], g = s[1], b = s[2];
#if SDL_BYTEORDER == SDL_BIG_ENDIAN
			d[0] = r;
			d[1] = g;
			d[2] = b;
			d[3] = has_alpha ? s[3] : 255;
#else
			d[0] = has_alpha ? s[3] : 255;
			d[1] = b;
			d[2] = g;
			d[3] = r;
#endif
		}
	}

	dst->bedata.p = tmp;

	return 0;
}

static void mbtk_sdl2_draw_free_pixmap(mbtk_display_t *display, mbtk_pixmap_t *px)
{
	free(px->bedata.p);
	px->bedata.p = NULL;
}

static mbtk_px_id_t mbtk_sdl2_draw_bind_pixmap(mbtk_window_t *win, mbtk_pixmap_t *px)
{
	mbtk_px_id_t pxid;
	pxid.p = px;
	pxid.l = 1; /* make it look valid */
	return pxid;
}

static void mbtk_sdl2_draw_pixmap(mbtk_window_t *win, int x, int y, mbtk_px_id_t pxid)
{
	sdl2_win_t *sw = win2sw(win);
	mbtk_pixmap_t *px = pxid.p;
	SDL_Rect rect;
	SDL_Texture *t;

	MAYBE_SKIP_RENDER(return);

	rect.x = 0; rect.y = 0;
	rect.w = px->w; rect.h = px->h;

	t = SDL_CreateTexture(sw->renderer, SDL_PIXELFORMAT_RGBA8888, SDL_TEXTUREACCESS_STATIC, px->w, px->h);
	SDL_UpdateTexture(t, &rect, px->bedata.p, px->w*4);

	if (px->has_alpha)
		SDL_SetTextureBlendMode(t, SDL_BLENDMODE_BLEND);

	rect.x = x; rect.y = y;
	SDL_RenderCopy(sw->renderer, t, NULL, &rect);
/*	SDL_DestroyTexture(t);*/
}

