static mbtk_window_t *mbtk_sdl2_id2win(Uint32 wid)
{
	SDL_Window *sdlw = SDL_GetWindowFromID(wid);
	if (sdlw == NULL)
		return NULL;
	return SDL_GetWindowData(sdlw, "w");
}

static int mbtk_sdl2_proc_stdin(SDL_Event ev, mbtk_event_t *dst)
{
	mbtk_display_t *disp = ev.user.data1;
/*	sdl2_display_t *sd = ev.user.data1;*/

	dst->type = MBTK_EV_RSTDIN;
	dst->win = disp->topwin;
	dst->data.rstdin.len = iot_stdio.len;
	dst->data.rstdin.data = iot_stdio.data;

/*	printf("STDIO EVENT %d '%s'!\n", iot_stdio.len, iot_stdio.data);*/

	SDL_SemTryWait(iot_stdio.enable); /* decrease to 0 to make sure the therad waits before sending the next event */
	if (!iot_stdio.disabled)
		SDL_SemPost(iot_stdio.enable); /* enable sending the next event */
	SDL_SemPost(iot_stdio.handled); /* release data */
	return 1;
}

static int mbtk_sdl2_proc_fdwatch(SDL_Event ev, mbtk_event_t *dst)
{
	mbtk_display_t *disp = ev.user.data1;
	sdl2_display_t *sd = ev.user.data1;

	dst->type = MBTK_EV_FDWATCH;
	dst->win = disp->topwin;
	dst->data.fdwatch.fd = sd->iot_fdwatch.fd;
	dst->data.fdwatch.readable = sd->iot_fdwatch.readable;
	dst->data.fdwatch.writable = sd->iot_fdwatch.writable;
	dst->data.fdwatch.closed = sd->iot_fdwatch.closed;

/*	printf("FDWATCH event fd=%d\n", dst->data.fdwatch.fd);*/

	SDL_SemPost(sd->iot_fdwatch.handled); /* release data */
	return 1;
}

static int mbtk_sdl2_proc_sdl2ev(SDL_Event ev, mbtk_event_t *dst)
{
	SDL_Event ev2;
	int r;

	switch(ev.type) {
		case SDL_WINDOWEVENT:
			switch(ev.window.event) {
				case SDL_WINDOWEVENT_EXPOSED:
					dst->type = MBTK_EV_WIN_EXPOSE;
					dst->win = mbtk_sdl2_id2win(ev.window.windowID);
					dst->data.win.x = dst->data.win.y = dst->data.win.w = dst->data.win.h = 0;
					return 1;
				case SDL_WINDOWEVENT_MOVED:
					dst->type = MBTK_EV_WIN_MOVE;
					dst->win = mbtk_sdl2_id2win(ev.window.windowID);
					dst->data.win.x = ev.window.data1;
					dst->data.win.y = ev.window.data2;
					dst->data.win.w = dst->data.win.h = 0;
					return 1;
				case SDL_WINDOWEVENT_RESIZED:
					dst->type = MBTK_EV_WIN_RESIZE;
					dst->win = mbtk_sdl2_id2win(ev.window.windowID);
					dst->data.win.x = dst->data.win.y = 0;
					dst->data.win.w = ev.window.data1;
					dst->data.win.h = ev.window.data2;
					return 1;
				case SDL_WINDOWEVENT_CLOSE:
					dst->type = MBTK_EV_WIN_CLOSE;
					dst->win = mbtk_sdl2_id2win(ev.window.windowID);
					dst->data.win.x = dst->data.win.y = dst->data.win.w = dst->data.win.h = 0;
					return 1;
				case SDL_WINDOWEVENT_ENTER:
					dst->type = MBTK_EV_WIN_ENTER;
					dst->win = mbtk_sdl2_id2win(ev.window.windowID);
					dst->data.win.x = dst->data.win.y = dst->data.win.w = dst->data.win.h = 0;
					return 1;
				case SDL_WINDOWEVENT_LEAVE:
					dst->type = MBTK_EV_WIN_LEAVE;
					dst->win = mbtk_sdl2_id2win(ev.window.windowID);
					dst->data.win.x = dst->data.win.y = dst->data.win.w = dst->data.win.h = 0;
					return 1;
			}
			break;

		case SDL_KEYUP:
		case SDL_KEYDOWN:
			dst->type = (ev.type == SDL_KEYDOWN) ? MBTK_EV_KEY_PRESS : MBTK_EV_KEY_RELEASE;
			dst->win = mbtk_sdl2_id2win(ev.key.windowID);
			mbtk_sdl2_key_xlate(dst, &ev);
			dst->data.key.edit = 0;
			/* check if there's a text input event available for the same keyup  and merge */
			if (ev.type == SDL_KEYUP) {
				r = SDL_PeepEvents(&ev2, 1, SDL_PEEKEVENT, SDL_FIRSTEVENT, SDL_LASTEVENT);
				if ((r == 1) && (ev2.type == SDL_TEXTINPUT) && (ev.key.timestamp == ev2.text.timestamp)) {
					SDL_PollEvent(&ev2);
					mbtk_sdl2_text_xlate(dst, &ev2);
				}
			}
			return 1;

		case SDL_TEXTINPUT:
			r = SDL_PeepEvents(&ev2, 1, SDL_PEEKEVENT, SDL_FIRSTEVENT, SDL_LASTEVENT);
			/* check if there's a keyup event available for the same keyup  and merge */
			if ((r == 1) && (ev2.type == SDL_KEYUP) && (ev.key.timestamp == ev2.text.timestamp)) {
				SDL_PollEvent(&ev2);
				dst->type = MBTK_EV_KEY_RELEASE;
				dst->win = mbtk_sdl2_id2win(ev.text.windowID);
				mbtk_sdl2_key_xlate(dst, &ev2);
				dst->data.key.edit = ev.text.text[0];
			}
			else {
				dst->type = MBTK_EV_KEY_RELEASE;
				dst->win = mbtk_sdl2_id2win(ev.text.windowID);
				dst->data.key.scancode = 0;
				dst->data.key.sym = 0;
				dst->data.key.mods = 0;
				mbtk_sdl2_text_xlate(dst, &ev);
			}
			return 1;

		case SDL_MOUSEBUTTONDOWN:
		case SDL_MOUSEBUTTONUP:
			{
				mbtk_window_t *win = mbtk_sdl2_id2win(ev.button.windowID);
				static mbtk_window_t *last_click_win;

				if (win == NULL)
					win = last_click_win;
				else
					last_click_win = win;

				dst->type = (ev.type == SDL_MOUSEBUTTONDOWN) ? MBTK_EV_MOUSE_PRESS : MBTK_EV_MOUSE_RELEASE;
				dst->win = win;
				if (win->entered) {
					dst->data.mouse.x = ev.button.x;
					dst->data.mouse.y = ev.button.y;
				}
				else
					dst->data.mouse.x = dst->data.mouse.y = -1;
				dst->data.mouse.button = mbtk_sdl2_mouse_button_xlate(ev.button.button);

				mbtk_display_set_mouse_button(win->display, dst->data.mouse.button, (ev.type == SDL_MOUSEBUTTONDOWN) ? 1 : 0);
			}
			return 1;

		case SDL_MOUSEMOTION:
			dst->type = MBTK_EV_MOUSE_MOTION;
			dst->win = mbtk_sdl2_id2win(ev.button.windowID);
			dst->data.mouse.x = ev.motion.x;
			dst->data.mouse.y = ev.motion.y;
			return 1;

		case SDL_MOUSEWHEEL:
			dst->type = MBTK_EV_MOUSE_SCROLL;
			dst->win = mbtk_sdl2_id2win(ev.button.windowID);
			dst->data.mouse.x = ev.wheel.x;
			dst->data.mouse.y = ev.wheel.y;
			return 1;

	}
	return 0;
}


static int mbtk_sdl2_iterate(mbtk_display_t *display, mbtk_event_t *dst, double timeout, int block)
{
	SDL_Event ev;

	do {
		int res;

		if (block || (timeout > 0))
			res = SDL_WaitEventTimeout(&ev, timeout*1000.0);
		else
			res = SDL_PollEvent(&ev);

		if (res) {
			if (ev.type == mbtk_sdl2_ev_rstdio)
				return mbtk_sdl2_proc_stdin(ev, dst);
			else if (ev.type == mbtk_sdl2_ev_fdwatch)
				return mbtk_sdl2_proc_fdwatch(ev, dst);
			else if (mbtk_sdl2_proc_sdl2ev(ev, dst))
				return 1;
		}
	} while(block);

	return 0;
}
