static long mbtk_sdl2_win_type(mbtk_window_type_t type)
{
	long base = 0;

	if (type & MBTK_WNTY_OPENGL) base = SDL_WINDOW_OPENGL;
	else if (type & MBTK_WNTY_VULKAN) base = SDL_WINDOW_VULKAN;

	switch(type & 0xff) {
		case MBTK_WNTY_NORMAL:         return base | SDL_WINDOW_RESIZABLE;
		case MBTK_WNTY_MODAL:          return base | SDL_WINDOW_RESIZABLE;
		case MBTK_WNTY_DROPDOWN_MENU:  return base | SDL_WINDOW_POPUP_MENU | SDL_WINDOW_ALWAYS_ON_TOP | SDL_WINDOW_MOUSE_CAPTURE;
		case MBTK_WNTY_POPUP_MENU:     return base | SDL_WINDOW_POPUP_MENU | SDL_WINDOW_ALWAYS_ON_TOP | SDL_WINDOW_MOUSE_CAPTURE;
		case MBTK_WNTY_TEAROFF_MENU:   return base | SDL_WINDOW_RESIZABLE;
		case MBTK_WNTY_COMBO:          return base | SDL_WINDOW_POPUP_MENU | SDL_WINDOW_ALWAYS_ON_TOP | SDL_WINDOW_MOUSE_CAPTURE;
		case MBTK_WNTY_TOOLTIP:        return base | SDL_WINDOW_TOOLTIP | SDL_WINDOW_ALWAYS_ON_TOP;
	}

	return base | SDL_WINDOW_RESIZABLE;
}

static mbtk_window_t *mbtk_sdl2_window_new(mbtk_display_t *disp, mbtk_window_type_t type, const char *title, long x, long y, long w, long h)
{
	mbtk_window_t *win = calloc(sizeof(mbtk_window_t), 1);
	sdl2_win_t *sw = (sdl2_win_t *)win;
	sdl2_display_t *sd = (sdl2_display_t *)disp;

	if ((x < 0) || (y < 0))
		x = y = SDL_WINDOWPOS_UNDEFINED;

	if ((w < 0) || (h < 0))
		w = h = 100;

	sw->window = SDL_CreateWindow(title, x, y, w, h, mbtk_sdl2_win_type(type));
	sw->renderer = SDL_CreateRenderer(sw->window, -1, 0 | ((getenv("MBTK_SW_RENDER") != NULL) ? SDL_RENDERER_SOFTWARE : SDL_RENDERER_ACCELERATED));
	SDL_SetRenderDrawBlendMode(sw->renderer, SDL_BLENDMODE_BLEND);
	SDL_SetWindowData(sw->window, "w", sw);
	sw->fonts = mbtk_sdl2_font_ctx_init(sw->renderer, &sd->font_cache);
	sw->font = NULL;

	return win;
}

static mbtk_window_t *mbtk_sdl2_subwindow_new(mbtk_display_t *disp, mbtk_widget_t *parent, long x, long y, long w, long h)
{
	mbtk_window_t *win = calloc(sizeof(mbtk_window_t), 1);
	return win;
}

static void mbtk_sdl2_window_close(mbtk_window_t *win)
{
	sdl2_win_t *sw = (sdl2_win_t *)win;
	SDL_DestroyWindow(sw->window);
}

static mbtk_window_t *mbtk_sdl2_win2render_win(mbtk_window_t *win)
{
	return (win->in_widget) ? win->in_widget->render_win : win;
}

static void mbtk_sdl2_window_set_title(mbtk_window_t *win, const char *title)
{
	sdl2_win_t *sw = (sdl2_win_t *)win;
	SDL_SetWindowTitle(sw->window, title);
}

static void mbtk_sdl2_window_raise(mbtk_window_t *win)
{
	sdl2_win_t *sw = (sdl2_win_t *)win;
	SDL_ShowWindow(sw->window);
	SDL_RaiseWindow(sw->window);
	/* TODO: may need to set input focus as well */
}

static void mbtk_sdl2_window_hide(mbtk_window_t *win)
{
	sdl2_win_t *sw = (sdl2_win_t *)win;
	SDL_HideWindow(sw->window);
}

static void mbtk_sdl2_window_show(mbtk_window_t *win)
{
	sdl2_win_t *sw = (sdl2_win_t *)win;
	SDL_ShowWindow(sw->window);
}

static void mbtk_sdl2_window_transient4(mbtk_window_t *win, mbtk_window_t *tr_for)
{
}

static void mbtk_sdl2_window_set_min_size(mbtk_window_t *win, long w, long h)
{
	sdl2_win_t *sw = (sdl2_win_t *)win;
	SDL_SetWindowMinimumSize(sw->window, w, h);
}

static void mbtk_sdl2_window_set_size(mbtk_window_t *win, long w, long h)
{
	sdl2_win_t *sw = (sdl2_win_t *)win;
	SDL_SetWindowSize(sw->window, w, h);
}

static int mbtk_sdl2_window_get_size(mbtk_window_t *win, long *w, long *h)
{
	sdl2_win_t *sw = (sdl2_win_t *)win;
	int wi, hi;

	SDL_GetWindowSize(sw->window, &wi, &hi);
	*w = wi;
	*h = hi;

	return 0;
}

