#include <X11/keysym.h>

static int mbtk_xlib_display_init(mbtk_backend_t *be, mbtk_display_t *disp, int *argc, char **argv[])
{
	xlib_display_t *xd = (xlib_display_t *)disp;

	mbtk_be_check_priv_sizes(xlib_display_t, xlib_win_t);

	xd->display = XOpenDisplay(NULL);
	if (xd->display == NULL)
		return -1;

	xd->screen = DefaultScreen(xd->display);
	xd->visual = DefaultVisual(xd->display, xd->screen);
	xd->depth = DefaultDepth(xd->display, xd->screen);
	xd->fd = ConnectionNumber(xd->display);
	xd->cmap = XDefaultColormap(xd->display, xd->screen);

	/* Look up and cache atoms */
	xd->xa.wmdel = XInternAtom(xd->display, "WM_DELETE_WINDOW", False);
	xd->xa.wm_hints = XInternAtom(xd->display, "_MOTIF_WM_HINTS", True);
	xd->xa.wintype = XInternAtom(xd->display, "_NET_WM_WINDOW_TYPE", False);
	xd->xa.wtype_dialog = XInternAtom(xd->display, "_NET_WM_WINDOW_TYPE_DIALOG", False);
	xd->xa.wtype_ddmenu = XInternAtom(xd->display, "_NET_WM_WINDOW_TYPE_DROPDOW_MENU", False);
	xd->xa.wtype_pumenu = XInternAtom(xd->display, "_NET_WM_WINDOW_TYPE_POPUP_MENU", False);
	xd->xa.wtype_menu = XInternAtom(xd->display, "_NET_WM_WINDOW_TYPE_MENU", False);
	xd->xa.wtype_combo = XInternAtom(xd->display, "_NET_WM_WINDOW_TYPE_COMBO", False);
	xd->xa.wtype_ttip = XInternAtom(xd->display, "_NET_WM_WINDOW_TYPE_TOOLTIP", False);
	xd->xa.wmst_modal = XInternAtom(xd->display, "_NET_WM_STATE_MODAL", False);
	xd->xa.wmstate = XInternAtom(xd->display, "_NET_WM_STATE", False);
	xd->xa.wmactw = XInternAtom(xd->display, "_NET_ACTIVE_WINDOW", False);
	xd->xa.clipboard = XInternAtom(xd->display, "CLIPBOARD", False);
	xd->xa.selection = XInternAtom(xd->display, "SELECTION", False);
	xd->xa.utf8_str = XInternAtom(xd->display, "UTF8_STRING", False);
	xd->xa.comp_str = XInternAtom(xd->display, "COMPOUND_STRING", False);
	xd->xa.targets = XInternAtom(xd->display, "TARGETS", False);
	xd->xa.savetargets = XInternAtom(xd->display, "SAVE_TARGETS", False);
	xd->xa.mult = XInternAtom(xd->display, "MULTIPLE", False);
	xd->xa.pair = XInternAtom(xd->display, "ATOM_PAIR", False);
	xd->xa.null = XInternAtom(xd->display, "NULL", False);

	xd->xk.lshift = XKeysymToKeycode(xd->display, XK_Shift_L);
	xd->xk.rshift = XKeysymToKeycode(xd->display, XK_Shift_R);
	xd->xk.lctrl = XKeysymToKeycode(xd->display, XK_Control_L);
	xd->xk.rctrl = XKeysymToKeycode(xd->display, XK_Control_R);
	xd->xk.lalt = XKeysymToKeycode(xd->display, XK_Alt_L);
	xd->xk.ralt = XKeysymToKeycode(xd->display, XK_Alt_R);

	vtp0_init(&xd->fonts);
	htip_init(&xd->id2win, longhash, longkeyeq);

	memset(&xd->watch, 0, sizeof(xd->watch));
	mbtk_watch_fd_set(&xd->watch, xd->fd, 1, 0);

	xd->clipboard = NULL;
	xd->base_time = mbtk_be_get_monotime();

	return 0;
}

static int mbtk_xlib_display_uninit(mbtk_backend_t *be, mbtk_display_t *disp)
{
	xlib_display_t *xd = (xlib_display_t *)disp;
	long n;

	mbtk_watch_fd_uninit(&xd->watch);

	/* fonts */
	for(n = 0; n < xd->fonts.used; n++)
		if (xd->fonts.array[n] != NULL)
			XFreeFont(xd->display, xd->fonts.array[n]);
	vtp0_uninit(&xd->fonts);
	htip_uninit(&xd->id2win);

	/* global X resources */
	XCloseDisplay(xd->display);
	return 0;
}

static double mbtk_xlib_get_time_el(mbtk_display_t *disp)
{
	xlib_display_t *xd = (xlib_display_t *)disp;
	return mbtk_be_get_monotime() - xd->base_time;
}

#define KEY_PRESSED(keycode) (!!(keys[keycode>>3] & (1<<(keycode&7))))
static mbtk_keymod_t mbtk_xlib_query_kbd_mods(mbtk_display_t *disp)
{
	xlib_display_t *xd = (xlib_display_t *)disp;
	char keys[32];
	mbtk_keymod_t res = 0;

	XQueryKeymap(xd->display, keys);
	if (KEY_PRESSED(xd->xk.lshift) ||KEY_PRESSED(xd->xk.rshift))
		res |= MBTK_KM_SHIFT;
	if (KEY_PRESSED(xd->xk.lctrl) ||KEY_PRESSED(xd->xk.rctrl))
		res |= MBTK_KM_CTRL;
	if (KEY_PRESSED(xd->xk.lalt) ||KEY_PRESSED(xd->xk.ralt))
		res |= MBTK_KM_ALT;

	return res;
}
