#define MBTK_NO_INCLUDE_WIDGETS
#include <libmbtk/widget.h>
#include <libmbtk/widget_imp.h>
#include <libmbtk/callback_imp.h>
#include <libmbtk/widget/hand_ev2cb.h>
#include <libmbtk/widget/canvas_native.h>

static int mbtk_canvas_native_fallback_minsize(mbtk_widget_t *w, long *mx, long *my)
{
	mbtk_canvas_native_t *canvas_native = (mbtk_canvas_native_t *)w;

	if (canvas_native->minsize != NULL)
		return canvas_native->minsize(canvas_native, mx, my);

	*mx = 1; *my = 1;
	return 0;
}

static int mbtk_canvas_native_draw_pre(mbtk_window_t *subwin, void *native_ctx)
{
	mbtk_canvas_native_t *canvas_native = (mbtk_canvas_native_t *)subwin->in_widget;

	canvas_native->draw(canvas_native, native_ctx, canvas_native->subwin->content_x0, canvas_native->subwin->content_y0, canvas_native->w.hvbox.w, canvas_native->w.hvbox.h);
	return 1; /* do not draw anything else for this subwin */
}

static void mbtk_canvas_native_post_assign(mbtk_widget_t *w)
{
	mbtk_canvas_native_t *canvas_native = (mbtk_canvas_native_t *)w;

	canvas_native->subwin = mbtk_subwindow_new(w->win->display, &canvas_native->w, 0, 0, 0, 0);
	mbtk_subwindow_assign(canvas_native->subwin, w->render_win);
	canvas_native->subwin->native_draw_pre = mbtk_canvas_native_draw_pre;
}

static void mbtk_canvas_native_pre_unassign(mbtk_widget_t *w)
{
	mbtk_canvas_native_t *canvas_native = (mbtk_canvas_native_t *)w;

	mbtk_subwindow_unassign(canvas_native->subwin);
	mbtk_subwindow_free(canvas_native->subwin);
	canvas_native->subwin = NULL;
	canvas_native->subwin->native_draw_pre = NULL;
}



static void mbtk_canvas_native_fallback_draw(mbtk_widget_t *w)
{
	mbtk_canvas_native_t *canvas_native = (mbtk_canvas_native_t *)w;

	/* ensure subwindow geometry matches widget's */
	mbtk_subwindow_geometry(canvas_native->subwin, w->hvbox.x, w->hvbox.y, w->hvbox.w, w->hvbox.h);

	if (canvas_native->draw != NULL) {
		mbtk_draw_start(canvas_native->subwin);
		/* this is where the backend code will call back the subwindows naive draw functions */
		mbtk_draw_flush(canvas_native->subwin);
	}
}

static void mbtk_canvas_native_fallback_draw_overlay(mbtk_widget_t *w)
{
}



static const mbtk_widget_imp_t mbtk_canvas_native_imp = {
	NULL, /* mbtk_canvas_native_destroy */

	mbtk_canvas_native_fallback_minsize,
	mbtk_canvas_native_fallback_draw,
	mbtk_canvas_native_fallback_draw_overlay,

	NULL,  /* get_prop */
	NULL,  /* set_prop */

	mbtk_canvas_native_post_assign,
	mbtk_canvas_native_pre_unassign
};

mbtk_canvas_native_t *mbtk_canvas_native_new(mbtk_canvas_native_t *caller_alloced)
{
	mbtk_canvas_native_t *canvas_native;
	mbtk_kw_t kw_canvas_native;

	mbtk_kw_cache(kw_canvas_native, "canvas_native");

	mbtk_create_wt(canvas_native, kw_canvas_native, &mbtk_canvas_native_imp, caller_alloced);

	canvas_native->w.relative_ev_coords = 1;
	mbtk_handler_ev2cb_install(&canvas_native->w);
	return canvas_native;
}

void mbtk_canvas_native_set_minsize_cb(mbtk_canvas_native_t *c, int (*minsize)(mbtk_canvas_native_t*, long*, long*))
{
	c->minsize = minsize;
}

void mbtk_canvas_native_set_draw_cb(mbtk_canvas_native_t *c, int (*draw)(mbtk_canvas_native_t *, void *, long, long, long, long))
{
	c->draw = draw;
}


void mbtk_canvas_native_callback_event(mbtk_canvas_native_t *c, mbtk_event_handled_t (*cb)(mbtk_widget_t *w, mbtk_kw_t id, mbtk_event_t *ev, void *user_data), void *user_data)
{
	mbtk_kw_t kw_event;
	mbtk_kw_cache(kw_event, "event");
	mbtk_callback_set_event(&c->w, kw_event, cb, user_data);
}
