#define MBTK_NO_INCLUDE_WIDGETS
#include <libmbtk/widget.h>
#include <libmbtk/widget_imp.h>
#include <libmbtk/draw.h>
#include <libmbtk/widget/progbar.h>
#include <libmbtk/widget/Common.h>


static void mbtk_progbar_imp_destroy(mbtk_widget_t *w) { }

#define FILLSIZE 4

static int mbtk_progbar_fallback_minsize(mbtk_widget_t *w, long *mx, long *my)
{
	long l, r, t, b;
	mbtk_window_t *win = w->win;
	const mbtk_style_t *style = mbtk_widget_style(w);
	mbtk_styleprop_t *sp = mbtk_styleprop_get(w);
	mbtk_widget_flags_t flags = w->flags;

	flags.activated = 1;
	style->min_box_frame_size(win, sp, flags, &l, &r, &t, &b);

	w->hvbox.fr_l = l; w->hvbox.fr_r = r;
	w->hvbox.fr_t = t; w->hvbox.fr_b = b;
	*mx = l+r+FILLSIZE; *my = t+b+FILLSIZE;

	return 0;
}

static void mbtk_progbar_calc(mbtk_widget_t *w, int draw, mbtk_arg_t *crds)
{
	mbtk_progbar_t *prb = (mbtk_progbar_t *)w;
	mbtk_window_t *win = w->win;
	const mbtk_style_t *style = mbtk_widget_style(w);
	mbtk_styleprop_t *sp = mbtk_styleprop_get(w);
	mbtk_widget_flags_t flags = w->flags;
	long l, r, t, b, bx1, by1, bx2, by2, bwi, bhe;
	long x1 = w->hvbox.x, y1 = w->hvbox.y, wi = w->hvbox.w, he = w->hvbox.h-1;
	double pos;
	mbtk_kw_t kw_hilight_color;

	/* clamp and normalize between 0 and 1 */
	if (prb->max > prb->min) {
		pos = prb->curr;
		if (pos < prb->min) pos = prb->min;
		if (pos > prb->max) pos = prb->max;
		pos = (pos  - prb->min) / (prb->max - prb->min);
	}
	else
		pos = 0;


	flags.activated = 1;
	style->min_box_frame_size(win, sp, flags, &l, &r, &t, &b);
	flags.activated = 0;
	if (draw)
		style->draw_bg_rect(win, sp, flags, x1, y1, wi, he);

	/* tune dimensions with the frame for the bar */
	bx1 = x1+l; by1 = y1+t;
	bwi = wi-(l+r);
	bhe = he-(t+b);
	bx2 = bx1 + ((double)bwi * pos) - 1;
	by2 = by1+bhe-1;

	if (draw) {
		mbtk_kw_cache(kw_hilight_color, "hilight.color");

		if (mbtk_widget_draw_set_style_color(w, kw_hilight_color) != 0)
			style->set_fg_color(win, sp, w->flags);
		mbtk_draw_fillrect(win, bx1, by1, bx2, by2);

		flags.activated = 1;
		style->draw_box_frame(win, sp, flags, x1, y1, wi, he);
	}

	if (crds != 0) {
		crds[0].type = MBTK_COORD; crds[0].v.c.x = x1;    crds[0].v.c.y = y1;
		crds[1].type = MBTK_COORD; crds[1].v.c.x = x1+wi; crds[1].v.c.y = y1+he;
		crds[2].type = MBTK_COORD; crds[2].v.c.x = bx2+1; crds[2].v.c.y = by1;
		crds[3].type = MBTK_COORD; crds[3].v.c.x = bx2;   crds[3].v.c.y = by2;
	}
}

static void mbtk_progbar_fallback_draw(mbtk_widget_t *w)
{
	mbtk_progbar_calc(w, 1, NULL);
}

static int mbtk_progbar_get_propa(mbtk_widget_t *w, mbtk_kw_t key, mbtk_arg_t *dst, int arrlen)
{
	mbtk_kw_t kw_range_val, kw_range_coords, kw_range_horiz;
	mbtk_progbar_t *prb = (mbtk_progbar_t *)w;

	mbtk_kw_cache(kw_range_val, "range.val");
	mbtk_kw_cache(kw_range_coords, "range.coords");
	mbtk_kw_cache(kw_range_horiz, "range.horiz");

	if ((key == kw_range_val) && (arrlen == 4)) {
		dst[0].type = MBTK_DOUBLE; dst[0].v.d = prb->min;
		dst[1].type = MBTK_DOUBLE; dst[1].v.d = prb->max;
		dst[2].type = MBTK_DOUBLE; dst[2].v.d = prb->curr;
		dst[3].type = MBTK_DOUBLE; dst[3].v.d = prb->curr;
		return 4;
	}

	if ((key == kw_range_coords) && (arrlen == 4)) {
		mbtk_progbar_calc(w, 0, dst);
		return 4;
	}

	if ((key == kw_range_horiz) && (arrlen == 1)) {
		dst[0].type = MBTK_BOOL; dst[0].v.b = 1;
		return 1;
	}

	return -1;
}

static int mbtk_progbar_set_propa(mbtk_widget_t *w, mbtk_kw_t key, const mbtk_arg_t *src, int arrlen)
{
	mbtk_kw_t kw_range_val, kw_range_coords;
	mbtk_progbar_t *prb = (mbtk_progbar_t *)w;

	mbtk_kw_cache(kw_range_val, "range.val");
	mbtk_kw_cache(kw_range_coords, "range.coords");

	if ((key == kw_range_val) && (arrlen == 4)) {
		if ((src[0].type != MBTK_DOUBLE) || (src[1].type != MBTK_DOUBLE) || (src[2].type != MBTK_DOUBLE) || (src[3].type != MBTK_DOUBLE))
			return -1;

		prb->min = src[0].v.d;
		prb->max = src[1].v.d;
		prb->curr = src[2].v.d;
		mbtk_widget_needs_redraw(w);
		return 4;
	}

	if ((key == kw_range_coords) && (arrlen == 4)) {
		long min, max, pos;

		mbtk_arg_t a[4];
		mbtk_progbar_calc(w, 0, a);
		min = a[0].v.c.x;
		max = a[1].v.c.x;
		pos = src[2].v.c.x;
		if (pos <= min)
			prb->curr = prb->min;
		else if (pos >= max)
			prb->curr = prb->max;
		else
			prb->curr = ((double)(pos - min) / (double)(max - min)) * (prb->max - prb->min) + prb->min;
		mbtk_widget_needs_redraw(w);
		return 4;
	}

	return -1;
}


static const mbtk_widget_imp_t mbtk_progbar_imp = {
	mbtk_progbar_imp_destroy,

	mbtk_progbar_fallback_minsize,
	mbtk_progbar_fallback_draw,
	mbtk_comm_frame_fallback_draw_overlay,

	mbtk_progbar_get_propa,
	mbtk_progbar_set_propa,

	NULL, /* post_assign */
	NULL  /* pre_unassign */
};

mbtk_progbar_t *mbtk_progbar_new(mbtk_progbar_t *caller_alloced)
{
	mbtk_progbar_t *progbar;
	mbtk_kw_t kw_progbar;

	mbtk_kw_cache(kw_progbar, "progbar");

	mbtk_create_wt(progbar, kw_progbar, &mbtk_progbar_imp, caller_alloced);
	progbar->min = 0.0;
	progbar->max = 1.0;
	progbar->curr = 0.0;
	progbar->w.hvbox.type = HVBOX_HBOX; /* don't count padding be part of the widget */

	return progbar;
}

int mbtk_progbar_set_curr(mbtk_progbar_t *prg, double curr, int clamp)
{
	if (curr < prg->min) {
		if (!clamp)
			return -1;
		prg->curr = prg->min;
	}
	else if (curr > prg->max) {
		if (!clamp)
			return -1;
		prg->curr = prg->max;
	}
	else
		prg->curr = curr;

	mbtk_widget_needs_redraw(&prg->w);
	return 0;
}
