#ifndef MBTK_TESTS_BE_INIT_H
#define MBTK_TESTS_BE_INIT_H

#include <string.h>
#include <libmbtk/libmbtk.h>
#include <libmbtk/backend/sdl2.h>
#include <libmbtk/backend/glfw.h>
#include <libmbtk/backend/xlib.h>
#include <libmbtk/backend/test.h>

char *backend_name;


/* Initialize be and disp from $BACKEND and argc/argv and return a font path array
   Exits on error. */
char **tests_backend_init(mbtk_backend_t *be, mbtk_display_t *disp, int *argc, char ***argv)
{
	char **font;
	int res;

	backend_name = getenv("BACKEND");
	if (backend_name == NULL)
		backend_name = "xlib";

#if HAVE_MBTK_BE_xlib
	if (strcmp(backend_name, "xlib") == 0) {
		static char *fnt[] = { "-*-helvetica-*-*-*-*-*-*-*-*-*-*-*-*" };
		font = fnt;
		res = mbtk_xlib_backend(be);
	}
#endif
#if HAVE_MBTK_BE_sdl2
	if (strcmp(backend_name, "sdl2") == 0) {
		static char *fnt[] = { "/usr/share/fonts/truetype/open-sans/OpenSans-Regular.ttf,size=18" };
		font = fnt;
		res = mbtk_sdl2_backend(be);
	}
#endif
#if HAVE_MBTK_BE_glfw
	if (strcmp(backend_name, "glfw") == 0) {
		static char *fnt[] = { "/usr/share/fonts/truetype/open-sans/OpenSans-Regular.ttf,size=18" };
		font = fnt;
		res = mbtk_glfw_backend(be);
	}
#endif
#if HAVE_MBTK_BE_test
	if (strcmp(backend_name, "test") == 0) {
		static char *fnt[] = { "dummy-font,key=val" };
		font = fnt;
		res = mbtk_test_backend(be);
	}
#endif

	if (res != 0) {
		fprintf(stderr, "Unable to set up backend %s\n", backend_name);
		exit(1);
	}

	if (mbtk_be_init(be) != 0) {
		fprintf(stderr, "Unable to initialize mbtk backend %s\n", backend_name);
		exit(1);
	}

	if (mbtk_display_init(be, disp, argc, argv) != 0) {
		fprintf(stderr, "Unable to initialize mbtk display with backend %s\n", backend_name);
		exit(1);
	}

	return font;
}

#endif
